package cn.iocoder.yudao.module.member.service.level;

import cn.hutool.core.bean.BeanUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.dal.dataobject.level.MemberUserLevelConfigDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserLevel.MemberUserLevelConfigMapper;
import cn.iocoder.yudao.module.member.enums.MemberLevelStatusEnum;
import cn.iocoder.yudao.module.member.service.levelLog.MemberUserLevelLogService;
import cn.iocoder.yudao.module.member.util.MenberUserLevelGenCodeUtils;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelConfigBaseRespVO;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelConfigReqVO;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelConfigUpdateVO;
import cn.iocoder.yudao.module.member.vo.userLevel.MemberUserLevelStatusReqVO;
import cn.iocoder.yudao.module.member.vo.userLevelLog.MemberUserLevelLogCreateReq;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 会员积分 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
@Lazy
public class MemberUserLevelConfigServiceImpl extends AbstractService<MemberUserLevelConfigMapper, MemberUserLevelConfigDO> implements MemberUserLevelConfigService {

    private final MemberUserLevelConfigMapper memberUserLevelConfigMapper;
    @Resource
    private  MenberUserLevelGenCodeUtils menberUserLevelGenCodeUtils;
    @Resource
    private MemberUserLevelLogService logService;
    @Resource
    private AdminUserApi adminUserApi;
    public MemberUserLevelConfigServiceImpl(MemberUserLevelConfigMapper memberUserLevelConfigMapper) {
        this.memberUserLevelConfigMapper = memberUserLevelConfigMapper;
    }

    @Override
    public PageResult<MemberUserLevelConfigBaseRespVO> getLevelPage(MemberUserLevelConfigReqVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        LambdaQueryWrapper<MemberUserLevelConfigDO> wrapper = Wrappers.lambdaQuery();
        if (query.getName() != null){
            wrapper.like(MemberUserLevelConfigDO::getName, query.getName());
        }
        if (query.getStatus() != null && !query.getStatus().isEmpty()){
            wrapper.in(MemberUserLevelConfigDO::getStatus, query.getStatus());
        }
        if (query.getLevel() != null && !query.getLevel().isEmpty()){
            wrapper.in(MemberUserLevelConfigDO::getLevel, query.getLevel());
        }
        if (query.getBeginTime() != null&& query.getEndTime() != null){
            wrapper.between(MemberUserLevelConfigDO::getEnableDate, query.getBeginTime(), query.getEndTime());
        }
        wrapper.eq(MemberUserLevelConfigDO::getDeleted, 0);
        Long total = memberUserLevelConfigMapper.selectCount(wrapper);
        wrapper.last(String.format("order by id desc limit %s, %s", start, size));
        List<MemberUserLevelConfigDO> list = this.list(wrapper);
        List<MemberUserLevelConfigBaseRespVO> result = list.stream().map(i -> BeanUtil.copyProperties(i, MemberUserLevelConfigBaseRespVO.class)).collect(Collectors.toList());
        return new PageResult<>(result, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public Boolean delete(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return true;
        }
        LambdaUpdateWrapper<MemberUserLevelConfigDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.in(MemberUserLevelConfigDO::getId, ids);
        wrapper.set(MemberUserLevelConfigDO::getDeleted, 1);
        return this.update(wrapper);
    }

    @Override
    public MemberUserLevelConfigBaseRespVO getMemberUserLevelConfig(Long id) {
        MemberUserLevelConfigDO memberUserLevelConfigDO = memberUserLevelConfigMapper.selectById(id);
        if (memberUserLevelConfigDO== null){
            throw exception(SCORE_RULE_NOT_EXISTS);
        }
        MemberUserLevelConfigBaseRespVO memberUserLevelConfigBaseRespVO = BeanUtil.copyProperties(memberUserLevelConfigDO, MemberUserLevelConfigBaseRespVO.class);
        String creator = adminUserApi.getUserName(memberUserLevelConfigDO.getCreator());
        memberUserLevelConfigBaseRespVO.setCreator(creator);
        String updater = adminUserApi.getUserName(memberUserLevelConfigDO.getUpdater());
        memberUserLevelConfigBaseRespVO.setUpdater(updater);
        return memberUserLevelConfigBaseRespVO;
    }

    @Override
    public Boolean add(MemberUserLevelConfigUpdateVO query) {
        validate(query);
        MemberUserLevelConfigDO memberUserLevelConfigDO = BeanUtil.copyProperties(query, MemberUserLevelConfigDO.class);
        //生成编号
        String ruleNumber = menberUserLevelGenCodeUtils.generateMemberLevelCode();
        memberUserLevelConfigDO.setRuleNumber(ruleNumber);
        //是否传入状态
        if (query.getStatus() != null && query.getStatus() == 1){
            memberUserLevelConfigDO.setStatus(query.getStatus());
            memberUserLevelConfigDO.setEnableDate(new Date());
        }else {
            memberUserLevelConfigDO.setStatus(0);
        }
        //插入日志
        boolean updateStatus = this.saveOrUpdate(memberUserLevelConfigDO);
        if (updateStatus){
            createLog(memberUserLevelConfigDO,1);
        }
        return updateStatus;
    }

    private void validate(MemberUserLevelConfigUpdateVO query) {
        if (query.getLowerCount() > query.getUpperCount()) {
            throw exception(LEVEL_BOUND_RANGE_ERROR);
        }
    }

    @Override
    public Boolean updateLevel(MemberUserLevelConfigUpdateVO query) {
        validate(query);
        MemberUserLevelConfigDO memberUserLevelConfigDO = BeanUtil.copyProperties(query, MemberUserLevelConfigDO.class);
        //插入日志
        boolean updateStatus = this.saveOrUpdate(memberUserLevelConfigDO);
        if (updateStatus){
            createLog(memberUserLevelConfigDO,2);
        }
        return updateStatus;
    }

    @Override
    public MemberUserLevelConfigBaseRespVO getLevelByCount(Integer scoreCount) {
        List<MemberUserLevelConfigDO> levelConfigList = this.list();
        List<MemberUserLevelConfigDO> sortLevelConfig = levelConfigList.stream()
                .sorted(Comparator.comparing(MemberUserLevelConfigDO::getLowerCount))
                .collect(Collectors.toList());
        MemberUserLevelConfigDO memberUserLevelConfigDO = sortLevelConfig.stream().filter(config -> config.getLowerCount() <= scoreCount && config.getUpperCount() >= scoreCount)
                .findAny().orElse(sortLevelConfig.size()!=0?sortLevelConfig.get(sortLevelConfig.size() - 1):null);
        return BeanUtil.copyProperties(memberUserLevelConfigDO, MemberUserLevelConfigBaseRespVO.class);
    }

    @Override
    public void updateStatus(MemberUserLevelStatusReqVO memberUserLevelStatusReqVO) {
        MemberUserLevelConfigDO memberUserLevelConfigDO =  memberUserLevelConfigMapper.selectById(memberUserLevelStatusReqVO.getId());
        if (memberUserLevelConfigDO == null) {
            throw exception(LEVEL_RULE_NOT_EXISTS);
        }
        Integer oldStatus = memberUserLevelConfigDO.getStatus();

        if (oldStatus == MemberLevelStatusEnum.ENABLED.getValue() && memberUserLevelStatusReqVO.getStatus() == MemberLevelStatusEnum.CLOSED.getValue()) {
            memberUserLevelConfigDO.setStatus(memberUserLevelStatusReqVO.getStatus());
        } else if (oldStatus == MemberLevelStatusEnum.DRAFT.getValue() && memberUserLevelStatusReqVO.getStatus() == MemberLevelStatusEnum.ENABLED.getValue()) {
            memberUserLevelConfigDO.setStatus(memberUserLevelStatusReqVO.getStatus());
            memberUserLevelConfigDO.setEnableDate(new Date());
        } else {
            throw exception(SCORE_RULE_STATUS_ERROR);
        }
        memberUserLevelConfigMapper.updateById(memberUserLevelConfigDO);
    }
    private void createLog(MemberUserLevelConfigDO memberUserLevelConfigDO,Integer operateType){
        MemberUserLevelLogCreateReq memberUserLevelLogcreateReq = MemberUserLevelLogCreateReq
                .builder().createTime(memberUserLevelConfigDO.getCreateTime())
                .configId(memberUserLevelConfigDO.getId())
                .operator(memberUserLevelConfigDO.getCreator())
                .remarks(null)
                .operateType(operateType)
                .bpmProcessId(null)
                .build();
        logService.createLevelLog(memberUserLevelLogcreateReq);
    }
}
